<?php
session_name('ATMPHPSESSID') ;
session_start() ;

$app = "aTinyMON Real-Time monitoring application for Asterisk" ;
$version = "v0.28 - 2007" ;

/** \mainpage aTinyMON Real-Time monitoring application for Asterisk
 * \version v0.28beta - 2007
 */

/* Written by Frederic LE FOLL (frederic       @     fr) __    /  \__
 * ___________________________          .lefoll free.      \__/      \_________ !
 *
 * This software is distributed under GPL 2 License (http://www.gnu.org/copyleft/gpl.html)
 */

/** \file
 * A Tiny real-time monitoring board for Asterisk.
 *
 * Debug modes :
 * 0 : no debug
 * 1 : PHP error_reporting = E_ALL, display_errors = On. Debug atinyman.pl
 * 2 : + time = ticket time, simplified session dump
 * 3 : + full session dump
 * 4 : + step-by-step
 */

include_once ('config.php') ;


/** Protect HTML code : add escape chars.
 *
 * @param $text Input String
 * @returns Converted String
 */
function my_htmlentities($text)
	{
	return (htmlentities($text, ENT_COMPAT, 'UTF-8')) ;
	}


/** Clean device status information : delete (obsolete) call information.
 *
 * In debug or demo mode, "now", i.e. reference for tickets obsolescence, is the timestamp
 * of last ticket extracted from DB.
 * Otherwise, "now" is current server time.
 *
 * @param $type Channel type (SIP, ...)
 * @param $dev Device
 * @param $forced Set to TRUE to delete ALL call information, FALSE to delete obsolete call information only
 * @returns Number of calls remaining
 */
function clean_dev_data($type, $dev, $forced)
	{
	$now = (DEBUG >= 2) || DEMO ? $_SESSION['last_ticket']['timestamp'] : time() ;
	$calls = 0 ;
	if (isset($_SESSION['st'][$type][$dev]['_chan']))
		{
		if ($forced)
			{
			$_SESSION['st'][$type][$dev]['_chan'] = array () ;
			}
		else
			{
			foreach (array_keys($_SESSION['st'][$type][$dev]['_chan']) as $chanid)
				{
				if ($now - $_SESSION['st'][$type][$dev]['_chan'][$chanid]['_ts'] > CALL_TIMEOUT)
					{
					unset ($_SESSION['st'][$type][$dev]['_chan'][$chanid]) ;
					}
				else
					{
					$calls++ ;
					}
				}
			}
		}
	if ($calls == 0 && $now - $_SESSION['st'][$type][$dev]['_ts'] > DEVICE_TIMEOUT)
		{
		unset ($_SESSION['st'][$type][$dev]) ;
		}
	return ($calls) ;
	}


/** Update device status information : handle an event about the device.
 *
 * @param $mode Mode : UPDATE or DELETE
 * @param $type Channel type (SIP, ...)
 * @param $dev Device
 * @param $ticket_a AMI ticket contents, stored in an associative array
 */
function update_dev($mode, $type, $dev, $ticket_a)
	{
	if (MASK_PSEUDO &&
	    ($type == 'Zap' && $dev == 'pseudo' ||
	     $type == 'OutgoingSpoolFailed'))
		return ;

	switch ($mode)
		{
	case UPDATE :
		if (!isset($_SESSION['st'][$type][$dev]))
			{
			$_SESSION['st'][$type][$dev] = array() ;
			}
		$_SESSION['st'][$type][$dev] = array_merge($_SESSION['st'][$type][$dev], $ticket_a) ;
		break ;

	case DELETE :
		unset ($_SESSION['st'][$type][$dev]) ;
		break ;

	default :
		}
	}


/** Parse "chantype/dev-chanid" string and extract fields.
 *
 * This function analyses the input string and extracts the fields.
 * It tries to make its best with < Z O M B I E > and suffixes alike,
 * as well as with AsyncGoto/ and possibly other prefixes.
 * Warnin : obviously, this simplification may involve a loss of information, or
 * even worse, may generate incorrect information !
 *
 * @param $cnxid String, supposedly looking like "chantype/dev-chanid"
 * @returns array (chantype, dev, chanid)
 */
function parse_cnxid($cnxid)
	{
	/* Remove any <xxx> suffix */
	$cnxid = preg_replace('/<\w+>$/', '', $cnxid) ;
	 /* Remove any xxx/ prefix ahead of "chantype/".
	  * Be even prepared to a cascade of A/B/C/... before chantype. */
	$cnxid = preg_replace('/^([\w\/]+\/)(\w+\/)/', '$2', $cnxid) ;
	list ($channel, $chanid) = explode('-', $cnxid) ;
	list($type, $dev) = explode("/", $channel) ;
	return (array ($type, $dev, $chanid)) ;
	}


/** Update device status information : handle an event about a call related to the device.
 *
 * Device inherits from call event timestamp (update last time we heard about
 * the device ... or about a related call).
 *
 * @param $mode Mode : UPDATE or DELETE
 * @param $cnxid Connection id with syntax "channeltype/device-channelid"
 * @param $ticket_a AMI ticket contents, stored in an associative array
 */
function update_dev_call($mode, $cnxid, $ticket_a)
	{
	list($type, $dev, $chanid) = parse_cnxid($cnxid) ;

	if (MASK_PSEUDO &&
	    ($type == 'Zap' && $dev == 'pseudo' ||
	     $type == 'OutgoingSpoolFailed'))
		return ;

	$_SESSION['st'][$type][$dev]['_ts'] = $ticket_a['_ts'] ;

	switch ($mode)
		{
	case UPDATE :
		if (!isset($_SESSION['st'][$type][$dev]['_chan'][$chanid]))
			{
			$_SESSION['st'][$type][$dev]['_chan'][$chanid] = array() ;
			}
		$_SESSION['st'][$type][$dev]['_chan'][$chanid] = array_merge($_SESSION['st'][$type][$dev]['_chan'][$chanid], $ticket_a) ;
		break ;

	case DELETE :
		if (!isset($_SESSION['st'][$type][$dev]['_chan']))
			{
			$_SESSION['st'][$type][$dev]['_chan'] = array() ;
			}
		if (isset($_SESSION['st'][$type][$dev]['_chan'][$chanid]))
			{
			unset ($_SESSION['st'][$type][$dev]['_chan'][$chanid]) ;
			}
		break ;

	default :
		}
	}

/** Elaborate aTinyMON display
 */
function run_session()
	{
	global $version, $ui, $status, $special, $db ;

	/* This code is called periodically, either because of refresh timeout, or because of user request.
	 * Check whether a user request is pending (Post or Get method). */

	$userrequest = FALSE ;
	if (isset($_REQUEST['submit']['quit']))
		{
		if (isset($_COOKIE[session_name()]))
	   		setcookie(session_name(), '', time() - 42000, '/') ;
		$_SESSION = array() ;
		session_destroy() ;
		header('Location: ' . REDIRECT) ;
		exit ;
		}
	if (isset($_REQUEST['submit']['freeze']))
		{
		$_SESSION['freeze'] = TRUE ;
		$userrequest = TRUE ;
		}
	if (isset($_REQUEST['submit']['refresh']))
		{
		$_SESSION['freeze'] = FALSE ;
		$userrequest = TRUE && !(DEBUG >= 4 /* Step-by-step mode */) ;
		}
	if (isset($_REQUEST['submit']['view']))
		{
		foreach (array_keys($_REQUEST['submit']['view']) as $type)
			{
			$_SESSION['view'][$type] = !$_SESSION['view'][$type] ;
			}
		$userrequest = TRUE ;
		}
	if (isset($_REQUEST['submit']['showoffline']))
		{
		$_SESSION['showoffline'] = !$_SESSION['showoffline'] ;
		$userrequest = TRUE ;
		}
	if (isset($_REQUEST['submit']['showevents']))
		{
		$_SESSION['showevents'] = !$_SESSION['showevents'] ;
		$userrequest = TRUE ;
		}
	if (isset($_REQUEST['itemsperrow']))
		{
		$_SESSION['itemsperrow'] = $_REQUEST['itemsperrow'] ;
		$userrequest = TRUE ;
		}
	if (isset($_REQUEST['zoom']))
		{
		$_SESSION['zoom'] = $_REQUEST['zoom'] ;
		$userrequest = TRUE ;
		}
	if (isset($_REQUEST['id']))
		{
		$_SESSION['last_ticket']['id'] = $_REQUEST['id'] ;
		$_SESSION['last_ticket']['datetime'] = NULL ;
		$_SESSION['last_ticket']['timestamp'] = NULL ;
		$_SESSION['last_ticket']['ticket'] = NULL ;
		$_SESSION['last_ticket']['handled'] = TRUE ;
		$userrequest = TRUE ;
		}

	/* Assign default value to any non initialized variable */

	if (!isset($_SESSION['counter']))
		{
		$_SESSION['counter'] = 0 ;
		}
	else
		{
		$_SESSION['counter']++ ;
		}
	if (!isset($_SESSION['freeze']))
		{
		$_SESSION['freeze'] = FALSE ;
		}
	if (!isset($_SESSION['online']))
		{
		$_SESSION['online'] = FALSE ;
		}
	if (!isset($_SESSION['refresh_min']))
		{
		$_SESSION['refresh_min'] = REFRESH_MIN ;
		}
	if (!isset($_SESSION['refresh_max']))
		{
		$_SESSION['refresh_max'] = REFRESH_MAX ;
		}
	if (!isset($_SESSION['last_ticket']))
		{
		$_SESSION['last_ticket']['id'] = 0 ;
		$_SESSION['last_ticket']['datetime'] = 0 ;
		$_SESSION['last_ticket']['timestamp'] = 0 ;
		$_SESSION['last_ticket']['ticket'] = '' ;
		$_SESSION['last_ticket']['handled'] = TRUE ;
		}
	if (!isset($_SESSION['st']))
		{
		$_SESSION['st'] = array () ;
		$_SESSION['st']['Asterisk']['status'] = 'unknown' ;
		}
	if (!isset($_SESSION['view']))
		{
		$_SESSION['view']['Asterisk'] = TRUE ;
		$_SESSION['view']['SIP'] = TRUE ;
		}
	if (!isset($_SESSION['itemsperrow']))
		{
		$_SESSION['itemsperrow'] = 6 ;
		}
	if (!isset($_SESSION['showoffline']))
		{
		$_SESSION['showoffline'] = OFFLINE_BY_DEFAULT ;
		}
	if (!isset($_SESSION['showevents']))
		{
		$_SESSION['showevents'] = EVENTS_BY_DEFAULT || DEBUG ;
		}
	if (!isset($_SESSION['hostname']))
		{
		$_SESSION['hostname'] = exec('hostname') ;
		}

	/* Connect to DB server */

	$db = mysql_pconnect(DB_HOST, DB_USERNAME, DB_PASSWORD) ;
	if (!$db)
		{
		trigger_error(mysql_error() . ' after mysql_pconnect()', E_USER_ERROR) ;
		}

	/* Select Database */

	if ($db)
		{
		$query = "set NAMES utf8" ;
		$result = $db ? mysql_query($query) : FALSE ; if (!$result && $db) trigger_error(mysql_error() . ' after : ' . $query, E_USER_ERROR) ;

		/* Add next two lines only if necessary */
		/* $query = "set CHARACTER_SET utf8" ; */
		/* $result = $db ? mysql_query($query) : FALSE ; if (!$result && $db) trigger_error(mysql_error() . ' after : ' . $query, E_USER_ERROR) ; */

		$result = mysql_select_db(DB_DATABASE) ;
		if (!$result)
			{
			trigger_error(mysql_error() . ' after mysql_select_db()', E_USER_ERROR) ;
			mysql_close($db) ;
			$db = NULL ;
			}
		}

	/* Check Tickets table (only during initialization phase) */

	if ($db && !$_SESSION['online'])
		{
		$query = "CREATE TABLE IF NOT EXISTS `tickets` (
		          `id` int(11) NOT NULL auto_increment,
		          `datetime` datetime NOT NULL default '0000-00-00 00:00:00',
		          `ticket` text NOT NULL,
		          PRIMARY KEY  (`id`)
		          ) DEFAULT CHARSET=utf8 ;" ;
		$result = mysql_query($query) ;
		if (!$result)
			{
			trigger_error(mysql_error() . ' after mysql_query()', E_USER_ERROR) ;
			mysql_close($db) ;
			$db = NULL ;
			}
		}

	/* Run background monitoring application */

	if ($db && !$_SESSION['online'])
		{
		if (DEMO)
			{
			$_SESSION['online'] = TRUE ;
			}
		else
			{
			$check = exec(MON_APP . ' ping') ;
			if ($check == 'pong')
				{
				$args = AST_USERNAME . ' ' . AST_PASSWORD .
				       ' -d=' . DEBUG .
				       ' -h=' . AST_HOST .
				       ' -p=' . AST_PORT .
				       ' -a' . /* Auto-reconnect to Asterisk */
				       ' -u' . /* Unique instance */
				       ' -de' . /* Detach */
				       ' -dbh=' . DB_HOST .
				       ' -dbp=' . DB_PORT .
				       ' -dbu=' . DB_USERNAME .
				       ' -dbw=' . DB_PASSWORD .
				       ' -dbd=' . DB_DATABASE .
				       ' -dbt=' . DB_TABLE .
				       ' -dbe=' . TICKETS_PURGE . /* Erase tickets older than ... */
				       // ' -dbc' . /* Create table if not exists */
				       ' -c="loop ' . POLLING_PERIOD . ' / action: sippeers / . / ! 1 / Action: zapshowchannels / . / ! 1 / Action: status / . /"' ;
				/* Playing with different ways to start a process in background, ...
				 * The proc_open(proc_close(...)) seems slightly prefered to popen(pclose(...)) and better anyway than exec(...)
				 * (my own experiments, and general opinions). */
				proc_close(proc_open (MON_APP . " $args &", array(), $pipes));
				$_SESSION['online'] = TRUE ;
				}
			else
				{
				trigger_error(MON_APP . " is not available (test result : '$check')", E_USER_ERROR) ;
				}
			}
		}

	$sid = SID ? "?" . strip_tags(SID) : "" ;
	$url = $_SERVER['PHP_SELF'] . $sid ;
	$now = time() ;
	$_SESSION['scanstart'] = $now ;

	/*********************************************************************************
	 * HTML Header, including refresh information.
	 *********************************************************************************/

	header("Cache-Control: no-cache, must-revalidate") ;
	header("Expires: Mon, 26 Jul 1997 05:00:00 GMT") ;
	header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT") ;
	if ($_SESSION['online'] && !$_SESSION['freeze'] && !(DEBUG >= 4 /* Step-by-step mode */))
		{
		$refresh_min = $_SESSION['refresh_min'] ;
		header("Refresh: $refresh_min; URL=$url");
		}

	print "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n" ;
	print "<html>\n" ;
	print "<head>\n" ;
	print "  <meta http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\">\n" ;
	print "  <title>atinymon - " . $_SESSION['hostname'] . "</title>\n" ;
	print "  <link type=\"text/css\" href=\"css/atinymon.css\" rel=\"stylesheet\">\n" ;
	print "</head>\n" ;
	print "<body>\n" ;

	/* Handle tickets available in DB.
	 * We may wait for new tickets within the time limit of $_SESSION['refresh_max'] */

	$maxtime = $now + $_SESSION['refresh_max'] - $_SESSION['refresh_min'] ;
	$newtickets = 0 ;                           /* Count the number of tickets during current page execution */
	$maxnewtickets = TICKETS_PER_REFRESH_MAX ;  /* Maximum number of ticket we want to read during current page execution */
	$nomoretickets = FALSE ;                    /* Flag : last DB query returned no ticket */
	$displaynow = FALSE ;                       /* Flag : stop reading tickets, and display current state */
	while (!$_SESSION['freeze'] &&              /* If display is user-frozen, don't update data */
	       !$userrequest &&                     /* If this is a page refresh because of user request, don't update data this time */
	       !($newtickets && $nomoretickets) &&  /* Flag : last DB query did not bring any ticket back */
	       $newtickets < $maxnewtickets &&      /* We dont' want to query indefinitely, even if there are MANY tickets ahead in DB */
	       time() <= $maxtime &&                /* Timeout */
	       !$displaynow)                        /* Flag : stop reading tickets, and display current state */ 
		{
		if ($nomoretickets)
			{
			/* If previous query returned no data, wait a second before retrying a query */
			sleep (1) ;
			}

		$ticketsavail = -1 ;
		$query = "select count(*) from `" . DB_TABLE . "` where `id`>" . $_SESSION['last_ticket']['id'] ;
		$result = $db ? mysql_query($query) : FALSE ; if (!$result && $db) trigger_error(mysql_error() . ' in : ' . $query, E_USER_ERROR) ;
		if ($result)
			{
			$num_rows = mysql_num_rows($result) ;
			if ($num_rows > 0)
				{
				$row = mysql_fetch_row($result) ;
				$ticketsavail = $row[0] ;
				}
			mysql_free_result($result) ;
			}
		else
			{
			$displaynow = TRUE ; /* Stop on error */
			}

		if ($ticketsavail > 0)
			{
			$nomoretickets = FALSE ;
			$newtickets++ ;
			$displaynow = $displaynow || (DEBUG >= 4) ; /* Step-by-step mode */
			$query = "select `id`, `datetime`, time_to_sec(timediff(`datetime`, now())), `ticket`from `" . DB_TABLE . "` where `id`>" . $_SESSION['last_ticket']['id'] . " limit 0, 1" ;
			$result = $db ? mysql_query($query) : FALSE ; if (!$result && $db) trigger_error(mysql_error() . ' in : ' . $query, E_USER_ERROR) ;
			if ($result)
				{
				$num_rows = mysql_num_rows($result) ;
				if ($num_rows > 0)
					{
					$row = mysql_fetch_row($result) ;
					$_SESSION['last_ticket']['id'] = $row[0] ;
					$_SESSION['last_ticket']['datetime'] = $row[1] ;
					$_SESSION['last_ticket']['timestamp'] = time() + $row[2] ;
					$_SESSION['last_ticket']['ticket'] = $row[3] ;
					$_SESSION['last_ticket']['handled'] = 0 ;
					}
				mysql_free_result($result) ;
				}
			else
				{
				$displaynow = TRUE ; /* Stop on error */
				}

			$ticket_l = explode("\n", $_SESSION['last_ticket']['ticket']) ;
			if (preg_match("/^Asterisk Call Manager\//", $ticket_l[0]))
				{
				array_shift($ticket_l) ;
				}
			$msg = trim($ticket_l[0]) ;
			if (strstr($msg, ': '))
				{
				list($msg, $indication) = explode(': ', $msg) ;
				}
			else
				{
				$indication = '' ;
				}

			/* Analyze ticket */

			switch ($msg)
				{

			case 'Response' :

				switch ($indication)
					{
				case 'Success' :
					$_SESSION['st']['Asterisk']['status'] = 'online' ;
					$_SESSION['last_ticket']['handled'] = 1 ;
					break ;

				case 'Goodbye' :
					$_SESSION['st']['Asterisk']['status'] = 'unknown' ;
					$_SESSION['last_ticket']['handled'] = 1 ;
					break ;

				default :

					}
				break ;

			case 'Event' :

				/* Asterisk is considered online by default as soon as we receive an event */
				$_SESSION['st']['Asterisk']['status'] = 'online' ;

				/* Make an associative array from ticket information */
				$ticket_a = array('_ts' => $_SESSION['last_ticket']['timestamp']) ; /* Store timestamp with ticket */
				foreach ($ticket_l as $line)
					{
					if (strstr($line, ': '))
						{
						$split = explode(": ", trim($line), 2) ;
						$item = $split[0] ;
						$value = count($split) > 1 ? $split[1] : '' ;
						$ticket_a[$item] = $value ;
						}
					}

				$_SESSION['last_ticket']['handled'] = TRUE ; /* switch 'default' entry will deny when required */
				switch ($indication)
					{
				case 'Shutdown' :
					$_SESSION['st'] = array () ;
					$_SESSION['st']['Asterisk']['status'] = 'offline' ;
					$displaynow = $displaynow || DEMO ;
					break ;

				case 'PeerEntry' :
					if (isset($ticket_a['Channeltype']) && isset($ticket_a['ObjectName']))
						{
						update_dev(UPDATE, $ticket_a['Channeltype'], $ticket_a['ObjectName'], $ticket_a) ;
						$type = $ticket_a['Channeltype'] ;
						$dev = $ticket_a['ObjectName'] ;
						if (isset($_SESSION['st'][$type][$dev]['IPaddress']) &&
						    $_SESSION['st'][$type][$dev]['IPaddress'] == '-none-')
							{
							clean_dev_data($type, $dev, 1) ;
							}
						}
					break ;

				case 'PeerStatus' :
					if (isset($ticket_a['Peer']))
						{
						list($type, $dev) = explode("/", $ticket_a['Peer']) ;
						update_dev(UPDATE, $type, $dev, $ticket_a) ;
						}
					break ;

				case 'Newchannel' :
				case 'Newcallerid' :
				case 'Newstate' :
				case 'Status' :
					if (isset($ticket_a['Channel']))
						{
						update_dev_call(UPDATE, $ticket_a['Channel'], $ticket_a) ;
						}
					$displaynow = $displaynow || DEMO && $indication != 'Newcallerid' ;
					break ;

				case 'Dial' :
					if (isset($ticket_a['Source']))
						{
						update_dev_call(UPDATE, $ticket_a['Source'], $ticket_a) ;
						}
					if (isset($ticket_a['Destination']))
						{
						update_dev_call(UPDATE, $ticket_a['Destination'], $ticket_a) ;
						}
					break ;

				case 'Hangup' :
					if (isset($ticket_a['Channel']))
						{
						update_dev_call(DELETE, $ticket_a['Channel'], $ticket_a) ;
						}
					$displaynow = $displaynow || DEMO ;
					break ;

				case 'MeetmeJoin' :
				case 'MeetmeLeave' :
					if (!isset($ticket_a['Meetme']) || !isset($ticket_a['Channel']))
						break ; /* Should not happen. Don't care. */
					$meetme = $ticket_a['Meetme'] ;
					$cnxid = $ticket_a['Channel'] ;
					list ($channel, $chanid) = explode('-', $cnxid) ;
					/* Update Meetme status. Let's mimic the usual "channeltype/device-channelid" identifier */
					if ($indication == 'MeetmeJoin')
						{
						update_dev_call(UPDATE, "Meetme/$meetme-$chanid", $ticket_a) ; 
						}
					else
						{
						update_dev_call(DELETE, "Meetme/$meetme-$chanid", $ticket_a) ;
						}
					/* Update also Participant device status */
					if (isset($ticket_a['Channel']))
						{
						update_dev_call(UPDATE, $ticket_a['Channel'], $ticket_a) ;
						}
					$displaynow = $displaynow || DEMO ;
					break ;

				case 'ZapShowChannels' :
					if (isset($ticket_a['Channel']))
						{
						update_dev(UPDATE, 'Zap', $ticket_a['Channel'], $ticket_a) ;
						}
					break ;

				case 'Rename' : /* This one is used for dark indicible reasons ... */
					if (!isset($ticket_a['Oldname']) || !isset($ticket_a['Newname']))
						break ; /* Should not happen. Don't care. */
					$o_cnxid = preg_replace('/<\w+>/', '', $ticket_a['Oldname']) ; /* Remove any <xxx> sequence */
					$n_cnxid = preg_replace('/<\w+>/', '', $ticket_a['Newname']) ; /* Remove any <xxx> sequence */
					if ($n_cnxid != $o_cnxid)
						{
						list($o_type, $o_dev, $o_chanid) = parse_cnxid($o_cnxid) ;
						list($n_type, $n_dev, $n_chanid) = parse_cnxid($n_cnxid) ;
						update_dev_call(UPDATE, $o_cnxid, array ()) ; /* Just to be sure that an entry exists */
						$_SESSION['st'][$n_type][$n_dev]['_chan'][$n_chanid] = $_SESSION['st'][$o_type][$o_dev]['_chan'][$o_chanid] ;
						$_SESSION['st'][$n_type][$n_dev]['_chan'][$n_chanid][$msg] = $indication ;
						unset ($_SESSION['st'][$o_type][$o_dev]['_chan'][$o_chanid]) ;
						}
					$displaynow = $displaynow || DEMO ;
					break ;

				case 'Alarm' :
					if (isset($ticket_a['Channel']))
						{
						update_dev(UPDATE, 'Zap', $ticket_a['Channel'], $ticket_a) ;
						}
					break ;

				case 'AlarmClear' :
					if (isset($ticket_a['Channel']))
						{
						$ticket_a['Alarm'] = 'No Alarm' ;
						update_dev(UPDATE, 'Zap', $ticket_a['Channel'], $ticket_a) ;
						}
					break ;

				case 'PeerlistComplete' :          /* End of an Events list, after a SipPeers query, ... */
				case 'StatusComplete' :            /* End of an Events list, after a Status query */
				case 'ZapShowChannelsComplete' :   /* End of an Events list, after a ZapShowChannels query */
				case 'Newexten' :                  /* Call routing progress information */
				case 'OriginateSuccess' :          /* Call originate success */
				case 'OriginateFailure' :          /* Call originate failure */
				case 'Link' :                      /* Communication established */
				case 'Unlink' :                    /* Communication discontinued */
				case 'Reload' :                    /* Asterisk configuration reloaded */
					/* aTinyMON just ignores these events. */
					break ;

				default :
					$_SESSION['last_ticket']['handled'] = FALSE ;
					}
				break ;

			default :

				}
			}
		else
			{
			$nomoretickets = TRUE ;
			}
		if (FREEZE_IF_NOT_HANDLED)
			{
			/* Auto-freeze if ticket not handled */
			$_SESSION['freeze'] = $_SESSION['freeze'] || !$_SESSION['last_ticket']['handled'] ;
			}
		}

	/*********************************************************************************
	 * aTinyMON page structure : simple table with two cells aside
	 *********************************************************************************/

	print "<table style=\"width: 100%;\">\n" ;
	print " <tbody>\n" ;
	print "  <tr>\n" ;

	/*********************************************************************************
	 * Left side : information, commands, snapshots/zooms
	 *********************************************************************************/

	print "   <td style=\"vertical-align: top; width: 200px;\">\n" ;

	/* Information */

	print "<span class=title>aTinyMON</span>" ;
	if ($_SESSION['freeze'])
		{
		print " <img src=\"" . $ui['frozen'][0] . "\" title=\"" . $ui['frozen'][1] . "\"/>" ;
		}
	else
		{
		print " <img src=\"" . $ui['running'][0] . "\" title=\"" . $ui['running'][1] . "\"/>" ;
		}
	print "<br/> <span class=nota>$version - Software under GPL license</span><br/>\n" ;

	print "aTinyMON : " . $_SESSION['hostname'] . "<br/>Asterisk : " . AST_HOST . "<br/><span class=\"info\">" . date("Y-m-d H:i:s", $now) . "</span><br/>\n" ;

	/* Commands */

	print "<form action=\"$url\" method=\"post\">\n" ;
	foreach (array_keys($_SESSION['view']) as $type)
		{
		print "<input type=\"image\" name=\"submit[view][$type]\" src=\"" .
	    	  ($_SESSION['view'][$type] ? IMG_SELECTED : IMG_NOT_SELECTED) .
		      "\"> $type<br/>\n" ;
		}

	if ($_SESSION['freeze'])
		{
		$rule_args_list = array(2, 4, 6, 8, 16) ;
		$preset = $_SESSION['itemsperrow'] ;
		print "<select name=\"itemsperrow\">" ;
		foreach ($rule_args_list as $arg)
			{
			$selected_status = ($arg == $preset) ? 'selected' : '' ;
			print "<option value=\"$arg\" $selected_status>$arg " ;
			}
		print "</select>" ;
		}
	else
		{
		print $_SESSION['itemsperrow'] ;
		}
	print " items/row<br/>" ;

	print "<br/>\n" ;
	print "<input type=\"image\" name=\"submit[freeze]\" src=\"" . $ui['freeze'][0] . "\" title=\"" . $ui['freeze'][1] . "\"/>\n" ;
	print "<img src=\"" . IMG_BLANK . "\">\n" ;
	print "<input type=\"image\" name=\"submit[refresh]\" src=\"" . $ui['refresh'][0] . "\" title=\"" . $ui['refresh'][1] . "\"/>\n" ;
	print "<img src=\"" . IMG_BLANK . "\">\n" ;
	print "<input type=\"image\" name=\"submit[quit]\" src=\"" . $ui['quit'][0] . "\" title=\"" . $ui['quit'][1] . "\"/>\n" ;
	print "<br/><br/>\n" ;

	/* Snapshots/zooms */

	if (isset($_SESSION['zoom']) && strstr($_SESSION['zoom'], '/'))
		{
		list($type, $dev) = explode ('/', $_SESSION['zoom']) ;
		print "<input type=\"submit\" name=\"zoom\" value=\"Close\"/> \n" ;
		print "<span class=\"info\">Zoom $type/$dev<br/>\n" ;
		print "<textarea name=\"ticket\" rows=\"10\" cols=\"32\" readonly>" ;
		if (is_array($_SESSION['st'][$type][$dev]))
			{
			foreach (array_keys($_SESSION['st'][$type][$dev]) as $attr)
				{
				if ($attr == '_chan' || $attr == '_ts')
					{
					continue ;
					}
				$value = $_SESSION['st'][$type][$dev][$attr] ;
				print my_htmlentities("$attr: $value\n") ;
				}
			}
		if (is_array($_SESSION['st'][$type][$dev]['_chan']))
			{
			foreach (array_keys($_SESSION['st'][$type][$dev]['_chan']) as $chan)
				{
				print "* Channel $chan\n" ;
				foreach (array_keys($_SESSION['st'][$type][$dev]['_chan'][$chan]) as $chanattr)
					{
					if ($chanattr == '_ts')
						{
						continue ;
						}
					$value = $_SESSION['st'][$type][$dev]['_chan'][$chan][$chanattr] ;
					print my_htmlentities("$chanattr: $value\n") ;
					}
				}
			}
		print "</textarea></span>" ;
		}

	else if (isset($_SESSION['last_ticket']))
		{
		print "<input type=\"image\" name=\"submit[showoffline]\" src=\"" .
    		  ($_SESSION['showoffline'] ? IMG_SELECTED : IMG_NOT_SELECTED) . "\"> Show offline devices<br/>\n" ;
		print "<input type=\"image\" name=\"submit[showevents]\" src=\"" .
    		  ($_SESSION['showevents'] ? IMG_SELECTED : IMG_NOT_SELECTED) . "\"> Show AMI events<br/>\n" ;
		print "<span class=\"info\">Last Ticket - ID " . $_SESSION['last_ticket']['id'] . (!$_SESSION['last_ticket']['handled'] ? " <span class=\"msg\"> Not handled </span>" : "") . "<br/>\n" ;
		print $_SESSION['last_ticket']['datetime'] . "<br/>\n" ;
		if ($_SESSION['showevents'])
			{
			print "<textarea name=\"ticket\" rows=\"10\" cols=\"32\" readonly>" . my_htmlentities($_SESSION['last_ticket']['ticket']) . "</textarea></span>" ;
			}
		}

	print "</form>" ;

	print "   </td>\n" ;

	/***********************************************************************
	 * Right side : real-time view
	 ***********************************************************************/

	print "   <td style=\"vertical-align: top;\">\n" ;

	$topofpage = TRUE ;
	$urlsnap = (strstr ($url, '?') ? $url . '&' : '?') . 'zoom=' ;

	ksort($_SESSION['st']) ;
	foreach (array_keys($_SESSION['st']) as $type)
		{
		if(!isset($_SESSION['view'][$type]))
			{
			/* Auto-create new items in the 'view' list according to new channels */
			$_SESSION['view'][$type] = VIEW_BY_DEFAULT ;
			ksort($_SESSION['view']) ;
			}

		if (!$_SESSION['view'][$type])
			{
			continue ;
			}

		if (!$topofpage)
			{
			print "<hr style=\"width: 100%; height: 2px;\">\n" ;
			}
		$topofpage = FALSE ;

		print "<div class=subtitle>$type</div>\n" ;
		switch ($type)
			{
		case 'Asterisk' :
			$sta = $_SESSION['st']['Asterisk']['status'] ;
			print "<table style=\"width: 100%;\"><tbody>\n<tr><td>" ;
			print "<img src=\"" . $status[$sta][0] . "\" alt=\"" . $status[$sta][1] . "\" title=\"" . $status[$sta][1] . "\">Asterisk \n" ;
			print "</td></tr></table>\n" ;
			break ;

		default :
			if (!isset($_SESSION['st'][$type]))
				{
				continue ;
				}

			$devs = array_keys($_SESSION['st'][$type]) ;
			sort($devs) ;
			print "<table style=\"width: 100%;\"><tbody>\n<tr>" ;
			$count = 0 ;
			$width = 100 / $_SESSION['itemsperrow'] ;
			foreach ($devs as $dev)
				{
				if ($count != 0 && $count % $_SESSION['itemsperrow'] == 0)
					{
					print "</tr>\n<tr>" ;
					}
				$channels   = clean_dev_data($type, $dev, 0) ;
				$registered = isset($_SESSION['st'][$type][$dev]) &&
				              ((isset($_SESSION['st'][$type][$dev]['IPaddress']) &&
				                $_SESSION['st'][$type][$dev]['IPaddress'] != '-none-') ||
				               (isset($_SESSION['st'][$type][$dev]['PeerStatus']) &&
				                $_SESSION['st'][$type][$dev]['PeerStatus'] != 'Unregistered' &&
				                $_SESSION['st'][$type][$dev]['PeerStatus'] != 'Unreachable')) ;
				$sta = $registered ? 'online' : 'offline' ;
				if ($channels)
					{
					/* Let's update device state according to its associated channels :
					 * try to differenciate "ringing" devices and "anything-else" devices. */
					foreach (array_keys($_SESSION['st'][$type][$dev]['_chan']) as $chanid)
						{
						if (!isset($_SESSION['st'][$type][$dev]['_chan'][$chanid]['State']))
							{
							$sta = 'active' ; /* Channel with no state indication : consider the device as 'active' by default */
							}
						else
							{
							switch ($_SESSION['st'][$type][$dev]['_chan'][$chanid]['State'])
								{
							case 'Down' :
								/* Just forget this channel at the moment */
								break ;
							case 'Ringing' :
								if ($sta != 'active') /* 'active' state has priority over 'ringing' */
									$sta = 'ringing' ;
								break ;
							default :
								$sta = 'active' ;
								}
							}
						}
					}
				if (isset($_SESSION['st'][$type][$dev]['Alarm']) &&
				    $_SESSION['st'][$type][$dev]['Alarm'] != 'No Alarm')
					{
					/* Alarm state overrides any other state */
					$sta = 'alarm' ;
					}
				if ($sta != 'offline' || $_SESSION['showoffline'])
					{
					print "<td style=\"width: $width%;\" vertical-align: top;>" ;
					print "<img src=\"" . (isset($special[$type][$sta]) ? $special[$type][$sta] : $status[$sta][0]) . 
					      "\" alt=\"" . $status[$sta][1] . "\" title=\"" . $status[$sta][1] . "\">" ;
					print "<a href=\"$urlsnap$type/$dev\">$dev</a>" ;
					print "</td>\n" ;
					$count++ ;
					}
				}
			while ($count++ % $_SESSION['itemsperrow'] != 0)
				{
				print "<td style=\"width: $width%;\" vertical-align: top;></td>\n" ;
				}
			print "</tr>\n" ;
			print "</tbody></table>\n" ;
			}
		}

	/* debug information */
	if (DEBUG >= 2)
		{
		print "<hr style=\"width: 100%; height: 2px;\"><class=\"info\"><pre>" ;
		if (DEBUG >= 3)
			{
			print_r($_SESSION) ;
			}
		else
			{
			foreach (array_keys($_SESSION) as $var)
				{
				print "$var = " ;
				if (!is_array($_SESSION[$var]))
					print $_SESSION[$var] . "\n" ;
				elseif (in_array($var, array ('st')))
				print "array ()\n" ;
				else
					print_r($_SESSION[$var]) ;
				}
			}
		print "</pre></class>\n" ;
		}

	print "   </td>\n" ;

	print "  </tr>\n" ;
	print " </tbody>\n" ;
	print "</table>\n" ;

	if (isset($_SESSION['errorlist']))
		{
		display_errors() ;
		}

	print "</body>\n" ;
	print "</html>\n" ;
	}


/** Store errors and syslog them. We'll display them alltogether later.
 */
function store_error($err_type, $err_msg, $err_file, $err_line)
	{
	/* PHP error type => syslog severity */
	$severity = array (
		E_ERROR =>           LOG_ERR,
		E_WARNING =>         LOG_WARNING,
		E_PARSE =>           LOG_ERR,
		E_NOTICE =>          LOG_NOTICE,
		E_CORE_ERROR =>      LOG_ERR,
		E_CORE_WARNING =>    LOG_WARNING,
		E_COMPILE_ERROR =>   LOG_ERR,
		E_COMPILE_WARNING => LOG_WARNING,
		E_USER_ERROR =>      LOG_ERR,
		E_USER_WARNING =>    LOG_WARNING,
		E_USER_NOTICE =>     LOG_NOTICE,
		E_STRICT =>          LOG_ERR
		) ;

	if (in_array($err_type, array (E_NOTICE, E_STRICT)))
		{
		return (1) ;
		}
	$_SESSION['errorlist'][] = array ($err_type, $err_msg, $err_file, $err_line) ;
	$priority = isset($severity[$err_type]) ? $severity[$err_type] : LOG_ERR ;
	syslog($priority, "aTinyMON ($err_file:$err_line) $err_msg") ;
	return (1) ;
	}


/** Display errors that have been stored.
 */
function display_errors()
	{
	/* PHP error typ => text */
	$type = array (
		E_ERROR =>           'ERROR',
		E_WARNING =>         'WARNING',
		E_PARSE =>           'PARSE',
		E_NOTICE =>          'NOTICE',
		E_CORE_ERROR =>      'CORE_ERROR',
		E_CORE_WARNING =>    'CORE_WARNING',
		E_COMPILE_ERROR =>   'COMPILE_ERROR',
		E_COMPILE_WARNING => 'COMPILE_WARNING',
		E_USER_ERROR =>      'USER_ERROR',
		E_USER_WARNING =>    'USER_WARNING',
		E_USER_NOTICE =>     'USER_NOTICE',
		E_STRICT =>          'STRICT'
		) ;
	$sid = SID ? "?" . strip_tags(SID) : '' ;
	$url = $_SERVER['PHP_SELF'] . $sid ;

	print "<div class=\"alertbox\">\n" ;
	foreach ($_SESSION['errorlist'] as $error)
		{
		list ($err_type, $err_msg, $err_file, $err_line) = $error ;
		$texttype = isset($type[$err_type]) ? $type[$err_type] : "Err $err_type" ;
		$file = preg_replace ('/.*\//', '', $err_file) ;
		print "$texttype ($file:$err_line) ". my_htmlentities($err_msg) . "<br/>\n" ;
		}
	/* Submit button */
	print "<br/><form action=\"$url\" method=\"post\">\n" ;
	print "<input type=\"submit\" name=\"close_errorlist\" value=\"close\"> " ;
	print "</form>\n" ;	
	print "</div>\n" ;
	
	$_SESSION['freeze'] = TRUE ;
	}

if (DEBUG > 0)
	{
	error_reporting(E_ALL) ;
	ini_set('display_errors', 'On') ;
	}
set_error_handler('store_error') ;
if (isset($_REQUEST['close_errorlist']))
	unset ($_SESSION['errorlist']) ;
$db = NULL ;
run_session() ;

?>
