<?php

/** \file
 * Settings and constants for aTinyMON
 */

/* Asterisk settings */

/** Asterisk host. */
define ('AST_HOST', 'localhost') ;
/** Asterisk Manager Interface TCP port (5038 usually). */
define ('AST_PORT', 5038) ;
/** Username for Asterisk Manager Interface.
 * This user must be defined in Asterisk manager.conf file. */
define ('AST_USERNAME', 'YOUR_AMI_USERNAME') ;
/** Password for Asterisk Manager Interface user. */
define ('AST_PASSWORD', 'YOUR_AMI_PASSWORD') ;

/* Database settings */

/** MySQL server host. */
define ('DB_HOST', 'localhost') ;
/** MySQL server port (3306 usually). */
define ('DB_PORT', 3306) ;
/** MySQL user for atinyman.pl and aTinyMON.php.
 * This user must have read/write permissions on the tickets table. */
define ('DB_USERNAME', 'YOUR_MYSQL_USERNAME') ;
/** Password for MySQL user. */
define ('DB_PASSWORD', 'YOUR_MYSQL_PASSWORD') ;
/** Database that hosts the tickets table ('test' database for instance). */
define ('DB_DATABASE', 'test') ;
/** Table that will contain the tickets.
 *     Suggestion to create tickets table :
 *         CREATE TABLE `tickets` (
 *           `id` int(11) NOT NULL auto_increment,
 *           `datetime` datetime NOT NULL default '0000-00-00 00:00:00',
 *           `ticket` text NOT NULL,
 *           PRIMARY KEY (`id`)
 *         ) ENGINE=InnoDB DEFAULT CHARSET=utf8 ;
 *     You may choose different ENGINE and DEFAULT CHARSET.
 *     aTinyMON/atinyman can create it automatically if they have sufficient permissions.
 */
define ('DB_TABLE', 'tickets') ;

/* Monitoring utility settings */

/** Location of atinyman.pl */
define ('MON_APP', '/var/www/html/atinymon/atinyman.pl') ;
/** Polling period for scheduled polling requests. */
define ('POLLING_PERIOD', 300) ;
/** How long to keep tickets.
 * Don't set this value too short, because TinyMON needs a bit of historical
 * data to self-train about Asterisk devices. Example : 900 = 15 minutes. */
define ('TICKETS_PURGE', 900) ;

/* Debug settings */

/** Debug mode : 1 or more will activate miscellaneous debugging features. */
define ('DEBUG', 0) ;
/** Auto-freeze mode : freeze display if an unknown event could not be handled. */
define ('FREEZE_IF_NOT_HANDLED', FALSE) ;
/** Demo mode : run aTinyMON with a demo database. */
define ('DEMO', FALSE) ;

/* Misc settings : HTML page refresh, ... */

/** Redirection when exiting aTinyMON. */
define ('REDIRECT', '..') ;
/** Delay before the web browser will request a page refresh. */
define ('REFRESH_MIN', 2) ;
/** Maximum delay between 2 pages. */
define ('REFRESH_MAX', 10) ;
/** How many tickets maximum should be handled for one page refresh. */
define ('TICKETS_PER_REFRESH_MAX', 1000) ;
/** How long should we wait before we "forget" a call when we have no news.
 * Advice : don't set lower than POLLING_PERIOD. */
define ('CALL_TIMEOUT', 3600) ;
/** How long should we wait before we "forget" a device when we have no news.
 * Advice : don't set lower than POLLING_PERIOD. */
define ('DEVICE_TIMEOUT', 7200) ;
/** Display by default all new channels types (SIP, IAX2, MGCP and others). */
define ('VIEW_BY_DEFAULT', TRUE) ;
/** Display offline devices by default. */
define ('OFFLINE_BY_DEFAULT', TRUE) ;
/** Display AMI events by default. */
define ('EVENTS_BY_DEFAULT', FALSE) ;
/** Mask pseudo-device Zap/pseudo (used for meetme conferences) and fake channel OutgoingSpoolFailed. */
define ('MASK_PSEUDO', TRUE) ;

/* Internal data : images, icons and tooltips */

define ('PATH_IMG', 'images/') ;
define ('IMG_BLANK', PATH_IMG . 'blank_16_16.png') ;
define ('IMG_SELECTED', PATH_IMG . 'nu/ok_16.png') ;
define ('IMG_NOT_SELECTED', PATH_IMG . 'nu/stop_16.png') ;

$ui = array(
	'frozen' =>  array (PATH_IMG . 'nu/ktimer.png', 'Frozen'),
	'running' => array (PATH_IMG . 'nu/misc.png', 'Running'),
	'refresh' => array (PATH_IMG . 'nu/tool_resume.png', 'Refresh'),
	'freeze' =>  array (PATH_IMG . 'nu/tool_pause.png',  'Freeze'),
	'quit' =>  array (PATH_IMG . 'nu/no.png',  'Close session')) ;

$status = array(
	'unknown' => array(PATH_IMG . '0purple22.png', 'Unknown'),
	'alarm' =>   array(PATH_IMG . '0red22.png', 'Offline'),
	'offline' => array(PATH_IMG . '0grey22.png', 'Offline'),
	'online' =>  array(PATH_IMG . '0blue22.png', 'Online'),
	'ringing' => array(PATH_IMG . '0orange22.png', 'Ringing'),
	'active' =>  array(PATH_IMG . '0green22.png', 'Active')) ;
$special['SIP'] = array(
	'unknown' => PATH_IMG . '1purple22.png',
	'alarm' =>   PATH_IMG . '1red22.png',
	'offline' => PATH_IMG . '1grey22.png',
	'online' =>  PATH_IMG . '1blue22.png',
	'ringing' => PATH_IMG . '1orange22.png',
	'active' =>  PATH_IMG . '1green22.png') ;
$special['IAX2'] = $special['SIP'] ;
$special['Meetme'] = array(
	'unknown' => PATH_IMG . '3purple22.png',
	'alarm' =>   PATH_IMG . '3red22.png',
	'offline' => PATH_IMG . '3grey22.png',
	'online' =>  PATH_IMG . '3blue22.png',
	'ringing' => PATH_IMG . '3orange22.png',
	'active' =>  PATH_IMG . '3green22.png') ;

/* Internal data : constants */
define ('UPDATE', 1) ;
define ('DELETE', 0) ;

?>
